@@ -12,15 +12,9 @@ import logging
 import aiohttp
 
 HAYWARD_API_URL = "https://app1.haywardomnilogic.com/HAAPI/HomeAutomation/API.ashx"
-# CONNECT_PARAMS = [
-#     UserName = "",
-#     Password = ""
-# ]
 
-# get_msp_config_file = {}
 _LOGGER = logging.getLogger("omnilogic")
 
-
 class OmniLogic:
     def __init__(self, username, password):
         self.username = username
@@ -98,7 +92,7 @@ class OmniLogic:
             try:
                 response = await resp.text()
             except aiohttp.ClientConnectorError as e:
-                raise HTTPException(e)
+                raise LoginException(e)
 
         responseXML = ElementTree.fromstring(response)
 
@@ -114,7 +108,7 @@ class OmniLogic:
         if methodName == "Login" and "There is no information" in response:
             # login invalid
             # response = {"Error":"Failed login"}
-            raise LoginException("Bad username or password")
+            raise LoginException("Failed Login: Bad username or password")
 
         if int(responseXML.find("./Parameters/Parameter[@name='Status']").text) != 0:
             self.request_statusmessage = responseXML.find(
@@ -133,7 +127,7 @@ class OmniLogic:
 
         if "There is no information" in response:
             raise OmniLogicException("Failure getting token.")
-            return '{"Error":"Failed"}'
+
         else:
             root = ElementTree.fromstring(response)
             userid = root[1][2].text
@@ -329,10 +323,10 @@ class OmniLogic:
 
                 mspconfig_list.append(configitem)
 
+            
             return mspconfig_list
         else:
             raise OmniLogicException("Failed getting MSP Config Data.")
-            return '{"Error":"Failed getting MSP Config Data."}'
 
     async def get_BOWS(self):
         # DEPRECATED - USE get_msp_config_data instead.
@@ -382,7 +376,6 @@ class OmniLogic:
                 alarmslist.append(site_alarms)
         else:
             raise OmniLogicException("Failure getting alarms.")
-            return {"Error": "Failure getting alarms."}
 
         return alarmslist
 
@@ -676,6 +669,7 @@ class OmniLogic:
         relays = []
         bow_lights = []
         bow_relays = []
+        bow_pumps = []
         bow_item = {}
 
         backyard_name = ""
@@ -692,6 +686,7 @@ class OmniLogic:
                 else:
                     BOW["Lights"] = bow_lights
                     BOW["Relays"] = bow_relays
+                    BOW["Pumps"] = bow_pumps
                     BOW_list.append(BOW)
                     backyard["BOWS"] = BOW_list
                     backyard_list.append(backyard)
@@ -716,6 +711,7 @@ class OmniLogic:
                 else:
                     BOW["Lights"] = bow_lights
                     BOW["Relays"] = bow_relays
+                    BOW["Pumps"] = bow_pumps
 
                     BOW_list.append(BOW)
 
@@ -763,9 +759,13 @@ class OmniLogic:
                 this_chlorinator = child.attrib
                 this_chlorinator["Name"] = bow_item["Chlorinator"]["Name"]
                 this_chlorinator["Shared-Type"] = bow_item["Chlorinator"]["Shared-Type"]
-                this_chlorinator["Operation"] = bow_item["Chlorinator"]["Operation"][
-                    "Chlorinator-Equipment"
-                ]
+                this_chlorinator["Operation"] = []
+
+                if type(bow_item["Chlorinator"]["Operation"]) == dict:
+                    this_chlorinator["Operation"].append(bow_item["Chlorinator"]["Operation"]["Chlorinator-Equipment"])
+                else:
+                    for equipment in bow_item["Chlorinator"]["Operation"]:
+                        this_chlorinator["Operation"].append(equipment)
 
                 BOW[child.tag] = this_chlorinator
 
@@ -784,13 +784,24 @@ class OmniLogic:
 
             elif child.tag == "Pump":
                 this_pump = child.attrib
-                this_pump["Name"] = bow_item["Pump"]["Name"]
-                this_pump["Type"] = bow_item["Pump"]["Type"]
-                this_pump["Function"] = bow_item["Pump"]["Function"]
-                this_pump["Min-Pump-Speed"] = bow_item["Pump"]["Function"]
-                this_pump["Max-Pump_Speed"] = bow_item["Pump"]["Function"]
 
-                BOW[child.tag] = this_pump
+                if type(bow_item["Pump"]) == dict:
+                  this_pump["Name"] = bow_item["Pump"]["Name"]
+                  this_pump["Type"] = bow_item["Pump"]["Type"]
+                  this_pump["Function"] = bow_item["Pump"]["Function"]
+                  this_pump["Min-Pump-Speed"] = bow_item["Pump"]["Min-Pump-Speed"]
+                  this_pump["Max-Pump-Speed"] = bow_item["Pump"]["Max-Pump-Speed"]
+                else:
+                  for pump in bow_item["Pump"]:
+                    #Find the right pump
+                    if pump["System-Id"] == this_pump["systemId"]:
+                      this_pump["Name"] = pump["Name"]
+                      this_pump["Type"] = pump["Type"]
+                      this_pump["Function"] = pump["Function"]
+                      this_pump["Min-Pump-Speed"] = pump["Min-Pump-Speed"]
+                      this_pump["Max-Pump_Speed"] = pump["Max-Pump-Speed"]
+
+                bow_pumps.append(this_pump)
 
             elif child.tag == "Heater":
                 this_heater = child.attrib
@@ -829,6 +840,7 @@ class OmniLogic:
 
         BOW["Lights"] = bow_lights
         BOW["Relays"] = bow_relays
+        BOW["Pumps"] = bow_pumps
         BOW_list.append(BOW)
 
         backyard["BOWS"] = BOW_list
@@ -848,7 +860,13 @@ class OmniLogic:
 
         if self.token != "" and len(self.systems) != 0:
             config_data = await self.get_msp_config_file()
-
+            
+            """
+            f = open("mspconfig_" + self.username + ".txt", "w")
+            f.write(str(config_data))
+            f.close()
+            """
+            
             for system in self.systems:
                 # Get the right instance of the ID for this system
                 config_item = {}
@@ -875,19 +893,27 @@ class OmniLogic:
                 ]
                 site_telem["Msp-Language"] = config_item["System"]["Msp-Language"]
                 site_telem["Unit-of-Measurement"] = config_item["System"]["Units"]
-                site_telem["Unit-of-Temperature"] = config_item["Backyard"]["Sensor"][
-                    "Units"
-                ]
+
+                if type(config_item["Backyard"]["Sensor"]) == dict:
+                  site_telem["Unit-of-Temperature"] = config_item["Backyard"]["Sensor"][
+                      "Units"
+                  ]
+                else:
+                  for sensor in config_item["Backyard"]["Sensor"]:
+                    if sensor["Name"] == "AirSensor":
+                      site_telem["Unit-of-Temperature"] = sensor["Units"]
 
                 telem_list.append(site_telem)
 
         else:
             raise OmniLogicException("Failure getting telemetry.")
-            return {"Error": "Failure getting telemetry."}
 
-        #f = open("telemetry.txt", "w")
-        #f.write(str(telem_list))
-        #f.close()
+        """
+        f = open("telemetry_" + self.username + ".txt", "w")
+        f.write(str(telem_list))
+        f.close()
+        """
+
         return telem_list
 
     # def get_alarm_list(self):
